#include-once
#include 'Include\Array.au3'
#include '_A3D_Geometry.au3'

Global Const $A3G_DEFAULT_COLOR = 0xFFE000DD
Global Const $A3G_TETRAHEDRON = 4
Global Const $A3G_HEXAHEDRON = 6
Global Const $A3G_OCTAHEDRON = 8
Global Const $A3G_DODECAHEDRON = 12
Global Const $A3G_ICOSAHEDRON = 20
Global Const $A3G_AXES = 1

; Geometry Format
; n = Anzahl Vertices, j = anzahl Polygone, k = anzahl Vertices in einem Polygon
; [0] - ($aVertices[n])[3]  -> in $aVertices[0] ist der erste Vertex im Format Array[3]
; [1] - ($aIndices[j])[k+1] -> in $aIndices[0] ist das erste Objekt bestehend aus k vertices im Format Array[k+1] mit Array[0] = k, und Array[1+] = Vertexindex
; [2] - $aIndexColors[n]    -> in $aIndexColors[0] ist die Farbe (0xAARRGGBB) zum ersten Polygon (flat-shading via brush)
; [3] - TO BE REMOVED
; [4] - $aMinMax[4]         -> BoundingBox (Minimalwert und Maximalwert für x, y, z)

Func _A3D_Polyhedron($iFaces = 4, $iVtxPerPoly = -1, $iColor = $A3G_DEFAULT_COLOR)
	Switch $iFaces
		Case 4 ; tet
			If $iVtxPerPoly = -1 Then $iVtxPerPoly = 3
			Return __A3G_Tetrahedron($iVtxPerPoly, $iColor)
		Case 6 ; hex
			If $iVtxPerPoly = -1 Then $iVtxPerPoly = 4
			Return __A3G_Hexahedron($iVtxPerPoly, $iColor)
		Case 8 ; oct
			If $iVtxPerPoly = -1 Then $iVtxPerPoly = 3
			Return __A3G_Octahedron($iVtxPerPoly, $iColor)
		Case 12 ; dodeca
			If $iVtxPerPoly = -1 Then $iVtxPerPoly = 5
			Return __A3G_Dodecahedron($iVtxPerPoly, $iColor)
		Case 20 ; icosa
			If $iVtxPerPoly = -1 Then $iVtxPerPoly = 3
			Return __A3G_Icosahedron($iVtxPerPoly, $iColor)
	EndSwitch
	Return ConsoleWrite('!ERR: _A3D_Polyhedron only knows platonic solids for 4, 6, 8, 12, 20 faces... ' & $iFaces & ' is not supported.' & @CRLF)
EndFunc

Func _A3D_Shape($iShapeID)
	Switch $iShapeID
		Case $A3G_AXES
			Return __A3G_ShapeAxes()
	EndSwitch
EndFunc

Func __A3G_ShapeAxes()
	Local $aGeo = [Array(Array(0, 0, 0), Array(1, 0, 0), Array(0, 1, 0), Array(0, 0, 1)), Array(Array(2, 0, 1), Array(2, 0, 2), Array(2, 0, 3)), Array(0xFFFF0000, 0xFF00FF00, 0xFF0000FF), 0]
	_A3G_UpdateMinMax($aGeo)
	Return $aGeo
EndFunc

Func __A3G_Tetrahedron($iVtxPerPoly = 3, $iColor = $A3G_DEFAULT_COLOR)
	If $iVtxPerPoly > 3 Then Return ConsoleWrite('ERROR: Cannot create tetrahedron with ' & $iVtxPerPoly & '-gons.'); geht nicht
	Local $_ = 1.0 / Sqrt(3.0), $aGeo[5] = [Array(Array(-$_, -$_, -$_), Array($_, -$_, $_), Array(-$_, $_, $_), Array($_, $_, -$_))]
	Switch $iVtxPerPoly
		Case 1
			$aGeo[1] = Array(Array(1, 0), Array(1, 1), Array(1, 2), Array(1, 3))
		Case 2
			$aGeo[1] = Array(Array(2, 0, 1), Array(2, 1, 3), Array(2, 2, 3), Array(2, 3, 0), Array(2, 1, 2), Array(2, 0, 2))
		Case 3
			$aGeo[1] = Array(Array(3, 0, 2, 3), Array(3, 2, 1, 3), Array(3, 1, 2, 0), Array(3, 0, 3, 1))
	EndSwitch
	_A3G_UpdateColor($aGeo, $iColor)
	_A3G_UpdateMinMax($aGeo)
	Return $aGeo
EndFunc

Func __A3G_Hexahedron($iVtxPerPoly = 4, $iColor = $A3G_DEFAULT_COLOR)
	If $iVtxPerPoly = 3 Or $iVtxPerPoly > 4 Then Return ConsoleWrite('ERROR: Cannot create heexahedron with ' & $iVtxPerPoly & '-gons.') ; geht nicht
	Local $_ = 1 / Sqrt(3), $aGeo[5] = [Array(Array(-$_, -$_, -$_), Array(-$_, -$_, $_), Array($_, -$_, $_), Array($_, -$_, -$_), Array(-$_, $_, $_), Array($_, $_, $_), Array($_, $_, -$_), Array(-$_, $_, -$_))]
	Switch $iVtxPerPoly
		Case 1
			$aGeo[1] = Array(Array(1, 0), Array(1, 1), Array(1, 2), Array(1, 3), Array(1, 4), Array(1, 5), Array(1, 6), Array(1, 7))
		Case 2
			$aGeo[1] = Array(Array(2, 0, 1), Array(2, 0, 3), Array(2, 0, 7), Array(2, 1, 2), Array(2, 1, 4), Array(2, 2, 3), Array(2, 2, 5), Array(2, 3, 6), Array(2, 4, 5), Array(2, 4, 7), Array(2, 5, 6), Array(2, 6, 7))
		Case 4
			$aGeo[1] = Array(Array(4, 1, 0, 3, 2), Array(4, 2, 3, 6, 5), Array(4, 0, 1, 4, 7), Array(4, 2, 5, 4, 1), Array(4, 6, 3, 0, 7), Array(4, 4, 5, 6, 7))
	EndSwitch
	_A3G_UpdateColor($aGeo, $iColor)
	_A3G_UpdateMinMax($aGeo)
	Return $aGeo
EndFunc

Func __A3G_Octahedron($iVtxPerPoly = 3, $iColor = $A3G_DEFAULT_COLOR)
	If $iVtxPerPoly > 3 Then Return ConsoleWrite('ERROR: Cannot create octahedron with ' & $iVtxPerPoly & '-gons.'); geht nicht
	Local $_ = 1, $aGeo[5] = [Array(Array($_, 0, 0), Array(0, $_, 0), Array(0, 0, $_), Array(-$_, 0, 0), Array(0, -$_, 0), Array(0, 0, -$_))]
	Switch $iVtxPerPoly
		Case 1
			$aGeo[1] = Array(Array(1, 0), Array(1, 1), Array(1, 2), Array(1, 3), Array(1, 4), Array(1, 5))
		Case 2
			$aGeo[1] = Array(Array(2, 0, 1), Array(2, 0, 2), Array(2, 0, 4), Array(2, 0, 5), Array(2, 1, 3), Array(2, 1, 5), Array(2, 1, 2), Array(2, 2, 3), Array(2, 2, 4), Array(2, 3, 4), Array(2, 3, 5), Array(2, 4, 5))
		Case 3
			$aGeo[1] = Array(Array(3, 0, 1, 2), Array(3, 0, 2, 4), Array(3, 0, 5, 1), Array(3, 0, 4, 5), Array(3, 1, 3, 2), Array(3, 1, 5, 3), Array(3, 2, 3, 4), Array(3, 3, 5, 4))
	EndSwitch
	_A3G_UpdateColor($aGeo, $iColor)
	_A3G_UpdateMinMax($aGeo)
	Return $aGeo
EndFunc

Func __A3G_Dodecahedron($iVtxPerPoly = 5, $iColor = $A3G_DEFAULT_COLOR)
	If $iVtxPerPoly = 3 Or $iVtxPerPoly = 4 Or $iVtxPerPoly > 5 Then Return ConsoleWrite('ERROR: Cannot create dodecahedron with ' & $iVtxPerPoly & '-gons.'); geht nicht
	Local $_ = (1.0 + Sqrt(5.0)) / 2.0, $a = $_ / Sqrt(3), $b = 1 / $_ / Sqrt(3) , $c = 1 / Sqrt(3), $aGeo[5] = [Array(Array($c, $c, $c), Array($c, $c, -$c), Array($c, -$c, $c), Array($c, -$c, -$c), Array(-$c, $c, $c), Array(-$c, $c, -$c), Array(-$c, -$c, $c), Array(-$c, -$c, -$c), Array(0, $a, $b), Array(0, $a, -$b), Array(0, -$a, $b), Array(0, -$a, -$b), Array($b, 0, $a), Array($b, 0, -$a), Array(-$b, 0, $a), Array(-$b, 0, -$a), Array($a, $b, 0), Array($a, -$b, 0), Array(-$a, $b, 0), Array(-$a, -$b, 0))]
	Switch $iVtxPerPoly
		Case 1
			$aGeo[1] = Array(Array(1, 0), Array(1, 1), Array(1, 2), Array(1, 3), Array(1, 4), Array(1, 5), Array(1, 6), Array(1, 7), Array(1, 8), Array(1, 9), Array(1, 10), Array(1, 11), Array(1, 12), Array(1, 13), Array(1, 14), Array(1, 15), Array(1, 16), Array(1, 17), Array(1, 18), Array(1, 19))
		Case 2
			$aGeo[1] = Array(Array(2, 0, 8), Array(2, 0, 12), Array(2, 0, 16), Array(2, 1, 9), Array(2, 1, 13), Array(2, 1, 16), Array(2, 2, 10), Array(2, 2, 12), Array(2, 2, 17), Array(2, 3, 11), Array(2, 3, 13), Array(2, 3, 17), Array(2, 4, 8), Array(2, 4, 14), Array(2, 4, 18), Array(2, 5, 9), Array(2, 5, 15), Array(2, 5, 18), Array(2, 6, 10), Array(2, 6, 14), Array(2, 6, 19), Array(2, 7, 11), Array(2, 7, 15), Array(2, 7, 19), Array(2, 8, 9), Array(2, 10, 11), Array(2, 12, 14), Array(2, 13, 15), Array(2, 16, 17), Array(2, 18, 19))
		Case 5
			$aGeo[1] = Array(Array(5, 0, 16, 1, 9, 8), Array(5, 16, 0, 12, 2, 17), Array(5, 0, 8, 4, 14, 12), Array(5, 13, 1, 16, 17, 3), Array(5, 15, 5, 9, 1, 13), Array(5, 2, 10, 11, 3, 17), Array(5, 12, 14, 6, 10, 2), Array(5, 7, 15, 13, 3, 11), Array(5, 18, 4, 8, 9, 5), Array(5, 6, 14, 4, 18, 19), Array(5, 19, 18, 5, 15, 7), Array(5, 7, 11, 10, 6, 19))
	EndSwitch
	_A3G_UpdateColor($aGeo, $iColor)
	_A3G_UpdateMinMax($aGeo)
	Return $aGeo
EndFunc

Func __A3G_Icosahedron($iVtxPerPoly = 3, $iColor = $A3G_DEFAULT_COLOR)
	If $iVtxPerPoly > 3 Then Return ConsoleWrite('ERROR: Cannot create icosahedron with ' & $iVtxPerPoly & '-gons.'); geht nicht
	Local $_ = (1.0 + Sqrt(5.0)) / 2.0, $l = Sqrt($_ * $_ + 1), $a = 1 / $l, $b = $_ / $l, $aGeo[5] = [Array(Array(0, -$a, -$b), Array(0, -$a, $b), Array(0, $a, -$b), Array(0, $a, $b), Array(-$a, -$b, 0), Array(-$a, $b, 0), Array($a, -$b, 0), Array($a, $b, 0), Array(-$b, 0, -$a), Array(-$b, 0, $a), Array($b, 0, -$a), Array($b, 0, $a))]
	Switch $iVtxPerPoly
		Case 1
			$aGeo[1] = Array(Array(1, 0), Array(1, 1), Array(1, 2), Array(1, 3), Array(1, 4), Array(1, 5), Array(1, 6), Array(1, 7), Array(1, 8), Array(1, 9), Array(1, 10), Array(1, 11))
		Case 2
			$aGeo[1] = Array(Array(2, 0, 2), Array(2, 0, 4), Array(2, 0, 6), Array(2, 0, 8), Array(2, 0, 10), Array(2, 1, 3), Array(2, 1, 4), Array(2, 1, 6), Array(2, 1, 9), Array(2, 1, 11), Array(2, 2, 5), Array(2, 2, 7), Array(2, 2, 8), Array(2, 2, 10), Array(2, 3, 5), Array(2, 3, 7), Array(2, 3, 9), Array(2, 3, 11), Array(2, 4, 6), Array(2, 4, 8), Array(2, 4, 9), Array(2, 5, 7), Array(2, 5, 8), Array(2, 5, 9), Array(2, 6, 10), Array(2, 6, 11), Array(2, 7, 10), Array(2, 7, 11), Array(2, 8, 9), Array(2, 10, 11))
		Case 3
			$aGeo[1] = Array(Array(3, 0, 2, 10), Array(3, 0, 4, 8), Array(3, 0, 6, 4), Array(3, 0, 8, 2), Array(3, 0, 10, 6), Array(3, 1, 3, 9), Array(3, 1, 11, 3), Array(3, 1, 9, 4), Array(3, 1, 4, 6), Array(3, 1, 6, 11), Array(3, 2, 5, 7), Array(3, 2, 8, 5), Array(3, 2, 7, 10), Array(3, 3, 7, 5), Array(3, 3, 5, 9), Array(3, 3, 11, 7), Array(3, 4, 9, 8), Array(3, 5, 8, 9), Array(3, 6, 10, 11), Array(3, 7, 11, 10))
	EndSwitch
	_A3G_UpdateColor($aGeo, $iColor)
	_A3G_UpdateMinMax($aGeo)
	Return $aGeo
EndFunc
